package com.fzm.admin.controller;

import cn.hutool.core.bean.BeanUtil;
import cn.hutool.core.lang.Assert;
import com.fzm.admin.params.AdminRoleParam;
import com.fzm.common.annotation.Authentication;
import com.fzm.common.entity.Admin;
import com.fzm.common.entity.AdminRole;
import com.fzm.common.enums.ResultCode;
import com.fzm.common.exception.GlobalException;
import com.fzm.common.interceptor.LoginUserInfo;
import com.fzm.common.model.ResponseModel;
import com.fzm.common.service.AdminRoleService;
import com.fzm.common.service.AdminService;
import com.github.pagehelper.PageInfo;
import io.swagger.annotations.Api;
import io.swagger.annotations.ApiOperation;
import io.swagger.annotations.ApiParam;
import lombok.RequiredArgsConstructor;
import org.apache.commons.lang3.StringUtils;
import org.springframework.beans.factory.annotation.Autowired;
import org.springframework.security.crypto.password.PasswordEncoder;
import org.springframework.web.bind.annotation.*;

import java.util.List;

/**
 * @author wt
 * @date 2022/3/11
 */
@Authentication
@RestController
@RequestMapping(value = "/admin/role")
@RequiredArgsConstructor(onConstructor = @__(@Autowired))
@Api(tags = "管理员权限管理")
public class AdminRoleController {
    private final AdminService adminService;
    private final AdminRoleService adminRoleService;
    private final PasswordEncoder passwordEncoder;

    @GetMapping("/pages")
    @ApiOperation(value = "分页查询")
    public ResponseModel<PageInfo<Admin>> pages(@ApiParam(value = "页码", required = true) @RequestParam Integer pageNum,
                                                @ApiParam(value = "每页记录数", required = true) @RequestParam Integer pageSize,
                                                @ApiParam(value = "用户名") @RequestParam(required = false) String username,
                                                @ApiParam(value = "昵称") @RequestParam(required = false) String nickname,
                                                @ApiParam(value = "角色类型") @RequestParam(required = false) Integer roleId) {
        if (roleId != null) {
            List<AdminRole> adminRoles = adminRoleService.listAll();
            Assert.isTrue(adminRoles.stream().map(AdminRole::getRoleId).anyMatch(id -> id.equals(roleId)), "请选择正确的权限ID");
        }
        PageInfo<Admin> pageInfo = adminService.pages(pageNum, pageSize, username, nickname, roleId);
        return ResponseModel.success(pageInfo);
    }

    @GetMapping("/list")
    @ApiOperation(value = "查询所有权限")
    public ResponseModel<List<AdminRole>> list() {
        return ResponseModel.success(adminRoleService.listAll());
    }

    @PostMapping("/add")
    @ApiOperation("新增权限")
    public ResponseModel<Object> add(@RequestBody AdminRoleParam adminRoleParam) {
        // 只有超级管理员能编辑权限
        Integer loginRoleId = LoginUserInfo.getLoginUser().getRoleId();
        if (loginRoleId != 1) {
            throw GlobalException.newException(ResultCode.FORBIDDEN);
        }

        String nickname = adminRoleParam.getNickname();
        Assert.notBlank(nickname, "昵称不能为空");

        String username = adminRoleParam.getUsername();
        Assert.notBlank(username, "用户名不能为空");
        Admin addUser = adminService.selectByUsername(username);
        Assert.isNull(addUser, "用户名已存在");

        String password = adminRoleParam.getPassword();
        Assert.notBlank(password, "密码不能为空");

        Integer roleId = adminRoleParam.getRoleId();
        Assert.notNull(roleId, "权限ID不能为空");
        List<AdminRole> adminRoles = adminRoleService.listAll();
        Assert.isTrue(adminRoles.stream().map(AdminRole::getRoleId).anyMatch(id -> id.equals(roleId)), "请选择正确的权限ID");

        Admin admin = BeanUtil.copyProperties(adminRoleParam, Admin.class);
        admin.setPassword(passwordEncoder.encode(password));
        admin.setNeedResetPassword(1);

        adminService.save(admin);

        return ResponseModel.success();
    }

    @PostMapping("/update")
    @ApiOperation("编辑权限")
    public ResponseModel<Object> update(@RequestBody AdminRoleParam adminRoleParam) {
        Integer loginRoleId = LoginUserInfo.getLoginUser().getRoleId();
        if (loginRoleId != 1) {
            throw GlobalException.newException(ResultCode.FORBIDDEN);
        }

        String username = adminRoleParam.getUsername();
        Assert.notBlank(username, "用户名不能为空");
        Admin updateUser = adminService.selectByUsername(username);
        Assert.notNull(updateUser, "用户名不存在");

        int needResetPassword = 0;
        String password = adminRoleParam.getPassword();
        if (StringUtils.isNotBlank(password)) {
            adminRoleParam.setPassword(passwordEncoder.encode(password));
            needResetPassword = 1;
        }

        Integer roleId = adminRoleParam.getRoleId();
        if (roleId != null) {
            List<AdminRole> adminRoles = adminRoleService.listAll();
            Assert.isTrue(adminRoles.stream().map(AdminRole::getRoleId).anyMatch(id -> id.equals(roleId)), "请选择正确的权限ID");
        }

        Integer freezeState = adminRoleParam.getFreezeState();
        if (freezeState != null) {
            Assert.isTrue(freezeState == 0 || freezeState == 1, "冻结状态码错误");
        }

        if (StringUtils.isAllBlank(adminRoleParam.getNickname(), password) && roleId == null && freezeState == null) {
            throw GlobalException.newException(ResultCode.VALIDATE_FAILED);
        }

        Admin admin = BeanUtil.copyProperties(adminRoleParam, Admin.class);
        admin.setNeedResetPassword(needResetPassword);

        adminService.updateByUserNameIf(admin);

        return ResponseModel.success();
    }

}
