package com.fzm.common.interceptor;

import cn.hutool.core.date.DateUtil;
import com.fzm.common.annotation.Authentication;
import com.fzm.common.constant.RedisConstant;
import com.fzm.common.constant.TokenConstant;
import com.fzm.common.entity.Admin;
import com.fzm.common.entity.User;
import com.fzm.common.enums.ResultCode;
import com.fzm.common.exception.GlobalException;
import com.fzm.common.service.AdminService;
import com.fzm.common.service.UserService;
import com.fzm.common.utils.JwtUtil;
import com.fzm.common.utils.RedisUtil;
import lombok.RequiredArgsConstructor;
import org.apache.commons.lang3.StringUtils;
import org.springframework.beans.factory.annotation.Autowired;
import org.springframework.web.method.HandlerMethod;
import org.springframework.web.servlet.HandlerInterceptor;
import org.springframework.web.servlet.ModelAndView;

import javax.servlet.http.HttpServletRequest;
import javax.servlet.http.HttpServletResponse;
import java.lang.reflect.Method;
import java.util.Date;

/**
 * @author tangtuo
 * @date 2021/7/13 14:30
 */
@RequiredArgsConstructor(onConstructor = @__(@Autowired))
public class AuthenticationInterceptor implements HandlerInterceptor {
    private final RedisUtil redisUtil;
    private final UserService userService;
    private final AdminService adminService;

    @Override
    public boolean preHandle(HttpServletRequest request, HttpServletResponse response, Object handler) throws Exception {
        // 如果不是映射到方法直接通过
        if (!(handler instanceof HandlerMethod)) {
            return true;
        }
        HandlerMethod handlerMethod = (HandlerMethod) handler;
        Method method = handlerMethod.getMethod();
        Class<?> cls = method.getDeclaringClass();
        // 先判断目标方法所在的类是否加了Authentication注解,再判断目标方法是否加了Authentication注解
        if (cls.isAnnotationPresent(Authentication.class) || method.isAnnotationPresent(Authentication.class)) {
            String token = request.getHeader("Authorization");
            if (StringUtils.isBlank(token)) {
                throw GlobalException.newException(ResultCode.UNAUTHORIZED, "请先登录");
            }
            if (!token.startsWith(TokenConstant.TOKEN_PREFIX)) {
                throw GlobalException.newException(ResultCode.UNAUTHORIZED, "非法token");
            }
            String realToken = token.substring(TokenConstant.TOKEN_PREFIX.length());
            if (StringUtils.isBlank(realToken) || "null".equals(realToken)) {
                throw GlobalException.newException(ResultCode.UNAUTHORIZED, "请先登录");
            }
            if (JwtUtil.isTokenExpired(token)) {
                throw GlobalException.newException(ResultCode.UNAUTHORIZED, "登录已过期");
            }
            String appId = JwtUtil.getAppIdFromToken(token);
            Integer userId = JwtUtil.getUserIdFromToken(token);

            String redisToken;
            String username;
            Integer roleId;
            if (TokenConstant.TOKEN_APP_ID_PORTAL.equals(appId)) {
                // h5用户
                User user = userService.getById(userId);
                if (user == null) {
                    throw GlobalException.newException(ResultCode.UNAUTHORIZED, "非法用户请求");
                }

                redisToken = redisUtil.get(RedisConstant.PORTAL_USER_TOKEN_PREFIX + userId);
                roleId = null;
                username = null;
            } else {
                // 后台管理用户
                Admin admin = adminService.getById(userId);
                if (admin == null) {
                    throw GlobalException.newException(ResultCode.UNAUTHORIZED, "非法用户请求");
                }

                // 判断冻结和更改密码
                Integer freezeState = admin.getFreezeState();
                if (freezeState != null && freezeState == 1) {
                    throw GlobalException.newException(ResultCode.FREEZE);
                }
                Integer needResetPassword = admin.getNeedResetPassword();
                String requestURI = request.getRequestURI();
                if (needResetPassword != null && needResetPassword == 1 && requestURI != null && !requestURI.contains("/admin/password")) {
                    throw GlobalException.newException(ResultCode.RESET_PASSWORD);
                }

                redisToken = redisUtil.get(RedisConstant.ADMIN_USER_TOKEN_PREFIX + userId);
                roleId = admin.getRoleId();
                username = admin.getUsername();
            }

            if (StringUtils.isBlank(redisToken)) {
                throw GlobalException.newException(ResultCode.UNAUTHORIZED, "登录已过期");
            }
            if (!redisToken.equals(realToken)) {
                Date date = JwtUtil.getCreateTimeFromToken(realToken);
                String format = DateUtil.format(date, "yyyy-MM-dd HH:mm:ss");
                throw GlobalException.newException(ResultCode.UNAUTHORIZED,
                        String.format("您的账号于 %s 登录另外一台设备,如非您本人所为,请立即修改密码", format));
            }
            // todo 认证完毕,对后台用户进行鉴权
            /*if (TokenConstant.TOKEN_APP_ID_ADMIN.equals(appId)) {
                Authentication annotation = method.getAnnotation(Authentication.class);
                String[] permissions = annotation.permissions();
                if (ArrayUtil.isNotEmpty(permissions)) {

                }
            }*/

            LoginUser loginUser = new LoginUser();
            loginUser.setUserId(userId);
            loginUser.setUsername(username);
            loginUser.setAppId(appId);
            loginUser.setRoleId(roleId);
            LoginUserInfo.setUserInfo(loginUser);
        }
        return true;
    }

    @Override
    public void postHandle(HttpServletRequest request, HttpServletResponse response, Object handler, ModelAndView modelAndView) throws Exception {

    }

    @Override
    public void afterCompletion(HttpServletRequest request, HttpServletResponse response, Object handler, Exception ex) throws Exception {
        LoginUserInfo.delUserInfo();
    }

    public static void main(String[] args) {
        String token = "Bearer null ";
        System.out.println(token.substring("Bearer ".length()));
        System.out.println(StringUtils.isBlank(token.substring("Bearer ".length())));
    }
}
