package com.fzm.common.service.impl;

import com.baomidou.mybatisplus.core.conditions.query.QueryWrapper;
import com.baomidou.mybatisplus.extension.service.impl.ServiceImpl;
import com.fzm.common.entity.Charge;
import com.fzm.common.entity.Order;
import com.fzm.common.entity.dto.OrderDto;
import com.fzm.common.entity.dto.OrderProcessMsg;
import com.fzm.common.entity.vo.OrderVo;
import com.fzm.common.enums.OrderStatus;
import com.fzm.common.enums.PayScene;
import com.fzm.common.enums.PayType;
import com.fzm.common.enums.ResultCode;
import com.fzm.common.exception.GlobalException;
import com.fzm.common.interceptor.LoginUserInfo;
import com.fzm.common.mapper.OrderMapper;
import com.fzm.common.service.*;
import com.fzm.common.utils.SnowflakeUtil;
import lombok.RequiredArgsConstructor;
import org.springframework.amqp.rabbit.core.RabbitTemplate;
import org.springframework.beans.factory.annotation.Autowired;
import org.springframework.stereotype.Service;
import org.springframework.transaction.annotation.Transactional;

import java.io.IOException;
import java.time.Duration;
import java.time.Instant;
import java.util.Date;
import java.util.List;

/**
 * @author tangtuo
 * @date 2022/1/19 15:42
 */
@Service
@Transactional(rollbackFor = RuntimeException.class)
@RequiredArgsConstructor(onConstructor = @__(@Autowired))
public class OrderServiceImpl extends ServiceImpl<OrderMapper, Order> implements OrderService {
    private final NftService nftService;
    private final CopyrightApplyService copyrightApplyService;
    private final SnowflakeUtil snowflakeUtil;
    private final RabbitTemplate rabbitTemplate;
    private final OrderMapper orderMapper;
    private final WxPayService wxPayService;
    private final ChargeService chargeService;

    @Override
    public Order getByPaySceneAndProductId(Integer payScene, Integer productId, Integer orderStatus) {
        QueryWrapper<Order> queryWrapper = new QueryWrapper<>();
        queryWrapper.eq("pay_scene", payScene)
                .eq("product_id", productId)
                .eq("order_status", orderStatus);
        return this.getOne(queryWrapper, false);
    }

    @Override
    public void updateOrderStatus(Long out_trade_no, OrderStatus orderStatus) {
        Order order = new Order();
        order.setId(out_trade_no);
        order.setOrderStatus(orderStatus.getStatus());
        this.updateById(order);
    }

    @Override
    public Order createOrder(OrderDto orderDto) {
        Charge charge = chargeService.getByType(orderDto.getPayScene());
        if (!orderDto.getFee().equals(charge.getFee())) {
            throw GlobalException.newException(ResultCode.FAILED, "订单创建失败, 金额有误");
        }
        Order successOrder = this.getByPaySceneAndProductId(orderDto.getPayScene(), orderDto.getProductId(), OrderStatus.PAYED.getStatus());
        if (successOrder != null) {
            throw GlobalException.newException(ResultCode.FAILED, "当前产品已支付成功,请勿重复创建订单");
        }
        Order payingOrder = this.getByPaySceneAndProductId(orderDto.getPayScene(), orderDto.getProductId(), OrderStatus.PAYING.getStatus());
        if (payingOrder != null) {
            throw GlobalException.newException(ResultCode.FAILED, "当前产品有一笔待支付订单,请勿重复创建");
        }
        Order order = new Order();
        long id = snowflakeUtil.snowflakeId();
        Integer productId = orderDto.getProductId();
        String orderName;
        Integer payScene = orderDto.getPayScene();
        if (PayScene.NFT.getCode().equals(payScene)) {
            orderName = nftService.getById(productId).getName();
        } else {
            orderName = copyrightApplyService.getById(productId).getOpusName();
        }
        order.setId(id);
        order.setOrderName(orderName);
        order.setPayScene(payScene);
        order.setOrderName(orderName);
        order.setFee(orderDto.getFee());
        order.setProductId(productId);
        order.setUserId(LoginUserInfo.getLoginUser().getUserId());
        order.setCreateDate(new Date());
        order.setUpdateDate(new Date());
        Integer orderStatus;
        if (orderDto.getFee() > 0) {
            orderStatus = OrderStatus.PAYING.getStatus();
        } else {
            // 订单金额为0的话,创建订单时,状态为支付成功
            orderStatus = OrderStatus.PAYED.getStatus();
            processOrder(id, productId, payScene);
        }
        order.setOrderStatus(orderStatus);
        this.save(order);
        return order;
    }

    @Override
    public void updatePayType(Long orderId, Integer payType) {
        Order o = new Order();
        o.setId(orderId);
        o.setPayType(payType);
        updateById(o);
    }

    @Override
    public Boolean cancel(Long orderId, OrderStatus orderStatus) {
        Order order = this.getById(orderId);
        Integer productId = order.getProductId();
        // 判断订单的类型
        if (PayScene.NFT.getCode().equals(order.getPayScene())) {
            // 删除nft的相关信息
            nftService.delete(productId);
        } else {
            // 删除版权申请的相关信息
            copyrightApplyService.delete(productId);
        }
        // 修改订单状态为取消
        this.updateOrderStatus(orderId, orderStatus);
        return true;
    }

    @Override
    public List<OrderVo> listOrder(Integer payScene, Integer orderStatus) {
        Integer userId = LoginUserInfo.getLoginUser().getUserId();
        return orderMapper.getByPayScene(payScene, userId, orderStatus);
    }

    @Override
    public void processOrder(Long orderId, Integer productId, Integer payScene) {
        // 订单支付成功后,给mq发送一条消息,处理nft发行或版权申请的状态
        OrderProcessMsg orderProcessMsg = new OrderProcessMsg(orderId, productId, payScene);
        rabbitTemplate.convertAndSend("order-exchange", "order.process", orderProcessMsg);
    }

    @Override
    public Boolean renewOrderStatus(Long orderId) throws IOException, InterruptedException {
        Order order = this.getById(orderId);
        if (order == null) {
            throw GlobalException.newException(ResultCode.DATA_ERROR, "unknown orderId: " + orderId);
        }
        if (PayType.WXPAY.getCode().equals(order.getPayType())) {
            wxPayService.queryOrder(orderId);
        }
        return true;
    }

    @Override
    public List<Order> getTimeOutOrders(int hours) {
        Instant instant = Instant.now().minus(Duration.ofHours(hours));
        QueryWrapper<Order> queryWrapper = new QueryWrapper<>();
        queryWrapper.eq("order_status", OrderStatus.PAYING.getStatus());
        queryWrapper.le("create_date", instant);
        return this.list(queryWrapper);
    }

}
